@@ -1,6 +1,6 @@
 package Pod::Weaver::Section::Contributors;
 {
-  $Pod::Weaver::Section::Contributors::VERSION = '0.001';
+  $Pod::Weaver::Section::Contributors::VERSION = '0.002';
 }
 use Moose;
 with 'Pod::Weaver::Role::Section';
@@ -11,41 +11,61 @@ use Moose::Autobox;
 use Pod::Elemental::Element::Nested;
 use Pod::Elemental::Element::Pod5::Verbatim;
 
+
 sub mvp_multivalue_args { qw( contributors ) }
 
 has contributors => (
     is      => 'ro',
     isa     => 'ArrayRef[Str]',
+    lazy    => 1,
     default => sub{ [] },
 );
 
+
 sub weave_section {
     my ($self, $document, $input) = @_;
+    my @contributors;
 
-    my $contributors = $self->contributors;
-    $contributors = $input->{contributors} if $input && $input->{contributors};
-    if ( $input && $input->{zilla} ) {
+    ## 1 - get contributors from $input parameter of weave_section()
+    push (@contributors, @{$input->{contributors}})
+        if $input->{contributors} && ref($input->{contributors}) eq 'ARRAY';
+
+    ## 2 - get contributors passed to Pod::Weaver::Section::Contributors
+    push (@contributors, @{$self->contributors});
+
+    ## 3 - get contributors passed to Dist::Zilla::Stash::PodWeaver
+    if ( $input->{zilla} ) {
         my $stash = $input->{zilla}->stash_named('%PodWeaver');
-        my $config;
+        my ($config, $contri);
         $config = $stash->get_stashed_config($self) if $stash;
-        $contributors = $config->{contributors}     if $config;
+        $contri = $config->{contributors}           if $config;
+        push (@contributors, @{$contri})         if $contri;
     }
 
-    return unless $contributors;
+    ## 4 - get contributors from source comments
+    my $ppi_document = $input->{ppi_document};
+    $ppi_document->find( sub {
+        my $ppi_node = $_[1];
+        if ($ppi_node->isa('PPI::Token::Comment') &&
+            $ppi_node->content =~ qr/^\s*#+\s*CONTRIBUTORS?:\s*(.+)$/m ) {
+            push (@contributors, $1);
+        }
+        return 0;
+    });
 
-    $contributors = [ $contributors ] unless ref $contributors;
-
-    return unless $contributors->length;
-
-    my $multiple_contributors = $contributors->length > 1;
+    ## 5 - remove repeated names, and sort them alphabetically
+    @contributors = List::MoreUtils::uniq (@contributors);
+    @contributors = sort (@contributors);
 
+    return unless @contributors;
+    my $multiple_contributors = @contributors > 1;
     my $name = $multiple_contributors ? 'CONTRIBUTORS' : 'CONTRIBUTOR';
 
-    my $result = $contributors->map(sub {
+    my $result = [map {
         Pod::Elemental::Element::Pod5::Ordinary->new({
             content => $_,
         }),
-    });
+    } @contributors];
 
     $result = [
         Pod::Elemental::Element::Pod5::Command->new({
@@ -75,7 +95,7 @@ sub weave_section {
 no Moose;
 1;
 
-
+__END__
 
 =pod
 
@@ -87,20 +107,27 @@ Pod::Weaver::Section::Contributors - a section listing contributors
 
 =head1 VERSION
 
-version 0.001
+version 0.002
 
 =head1 SYNOPSIS
 
-dist.ini:
+on dist.ini:
 
     [PodWeaver]
     [%PodWeaver]
-    Contributors.contributors[0] = Keedi Kim (KEEDI)
-    Contributors.contributors[1] = Jeen Lee (JEEN)
+    Contributors.contributors[0] = keedi - Keedi Kim - 김도형 (cpan: KEEDI) <keedi@cpan.org>
+    Contributors.contributors[1] = carandraug - Carnë Draug (cpan: CDRAUG) <cdraug@cpan.org>
 
-weaver.ini:
+and/or weaver.ini:
 
     [Contributors]
+    contributors = keedi - Keedi Kim - 김도형 (cpan: KEEDI) <keedi@cpan.org>
+    contributors = carandraug - Carnë Draug (cpan: CDRAUG) <cdraug@cpan.org>
+
+and/or in the source of individual files:
+
+    # CONTRIBUTOR:  keedi - Keedi Kim - 김도형 (cpan: KEEDI) <keedi@cpan.org>
+    # CONTRIBUTORS: carandraug - Carnë Draug (cpan: CDRAUG) <cdraug@cpan.org>
 
 =head1 DESCRIPTION
 
@@ -108,11 +135,19 @@ This section adds a listing of the documents contributors.  It expects a C<contr
 input parameter to be an arrayref of strings.  If no C<contributors> parameter is
 given, it will do nothing.  Otherwise, it produces a hunk like this:
 
-  =head1 CONTRIBUTORS
+    =head1 CONTRIBUTORS
 
     Contributor One <a1@example.com>
     Contributor Two <a2@example.com>
 
+To support distributions with multiple modules, it is also able to derive a list
+of contributors in a file basis by looking at comments on each module. Names of
+contributors on the source, will only appear on the POD of those modules.
+
+=for Pod::Coverage mvp_multivalue_args
+
+=for Pod::Coverage weave_section
+
 =head1 SEE ALSO
 
 =over
@@ -131,14 +166,15 @@ given, it will do nothing.  Otherwise, it produces a hunk like this:
 
 Keedi Kim - 김도형 <keedi@cpan.org>
 
+=head1 CONTRIBUTOR
+
+carandraug - Carnë Draug (cpan: CDRAUG) <cdraug@cpan.org>
+
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2012 by Keedi Kim.
+This software is copyright (c) 2013 by Keedi Kim.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
 
 =cut
-
-
-__END__
